from pdcom5.PdComWrapper import Process, Subscriber, Transmission, Subscription
import socket


class MySub(Subscriber):
    def __init__(self, mode, proc, path):
        super().__init__(mode)
        self._sub1 = Subscription(self, proc, path)

    def stateChanged(self, s: Subscription):
        print("State changed:", s.state)

    def newValues(self, time):
        print("New Values:", self._sub1.value)


class MyProc(Process):
    def __init__(self, host, port):
        super().__init__()
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._sock.connect((host, port))
        self._running = True
        self._connected = False

    def write(self, b):
        self._sock.sendall(b)

    def read(self, b):
        return self._sock.recv_into(b)

    def connected(self):
        print("Yaay, connected!")
        self._connected = True

    def listReply(self, vars, dirs):
        print("Vars:")
        for var in vars:
            print("  ", var.path)
        print("Dirs:")
        for dir in dirs:
            print("  ", dir)


if __name__ == "__main__":
    p = MyProc("localhost", 2345)
    while not p._connected:
        p.asyncData()
    sub = MySub(Transmission.event_mode, p, "/input/klemme1")
    p.list()
    try:
        while p._running:
            p.asyncData()
    except KeyboardInterrupt:
        # need to unsubscribe before Process goes away
        sub._sub1.cancel()
        pass
