/*****************************************************************************
 *
 * Copyright (C) 2021 Bjarne von Horn (vh at igh dot de).
 *
 * This file is part of the PdCom library.
 *
 * The PdCom library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The PdCom library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the PdCom library. If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#include <iostream>
#include <pdcom5/PosixProcess.h>
#include <pdcom5/Process.h>
#include <pdcom5/SimpleLoginManager.h>
#include <pdcom5/Variable.h>
#include <sasl/sasl.h>


class MyProcess : public PdCom::Process, private PdCom::PosixProcess
{
    int read(char *buf, int count) override { return posixRead(buf, count); }
    void write(const char *buf, size_t count) override
    {
        posixWriteBuffered(buf, count);
    }
    void flush() override { posixFlush(); }

    void listReply(
            std::vector<PdCom::Variable> vars,
            std::vector<std::string> /* directories */) override
    {
        for (const auto &v : vars)
            std::cout << "Found var " << v.getPath() << "\n";
        running_ = false;
    }

    void connected()
    {
        std::cout << "Connected!" << std::endl;
        connected_ = true;
    }

  public:
    MyProcess(const char *host = "127.0.0.1") :
        PdCom::Process(), PdCom::PosixProcess(host, 2345)
    {}

    bool running_   = true;
    bool connected_ = false;
};

class MyAuthManager : public PdCom::SimpleLoginManager
{
  public:
    PdCom::Process *p_ = nullptr;
    MyAuthManager() : PdCom::SimpleLoginManager("localhost") {}
    std::string getAuthname() override { return "bjarne"; }
    std::string getPassword() override { return "test"; }

    void completed(bool success) override
    {
        std::cout << "completed: " << success << std::endl;
        if (success) {
            p_->list("");
        }
    }
};

int main()
{
    if (sasl_client_init(nullptr) != SASL_OK)
        return -1;
    MyAuthManager am;
    MyProcess p;
    p.setAuthManager(&am);
    am.p_ = &p;
    while (!p.connected_)
        p.asyncData();
    am.login();
    while (p.running_) {
        p.asyncData();
    }
    am.logout();
    p.asyncData();
    return 0;
}
